import { getNamedFormat, filterProps, IntlError, IntlErrorCode, createIntl as createIntl$1, MissingDataError } from '@formatjs/intl';
export { createIntlCache } from '@formatjs/intl';
import { SupportedLocales, CanonicalizeLocaleList, ComputeExponent, FormatNumericToString } from '@formatjs/ecma402-abstract';
import { ResolveLocale } from '@formatjs/intl-localematcher';

const localeData = Object.create(null);
const availableLocales = new Set();
let defaultLocale = null;
function getDefaultLocale() {
  if (defaultLocale == null) {
    throw new Error('No compact number data has been loaded');
  }
  return defaultLocale;
}
function setDefaultLocaleDataLocale(value) {
  let apply = value;
  if (!availableLocales.has(apply)) {
    apply = new Intl.Locale(apply).minimize().toString();
    if (apply === value) {
      throw new Error(`No compact number data has been loaded for locale "${value}"`);
    }
    if (!availableLocales.has(apply)) {
      throw new Error(`No compact number data has been loaded for locale "${value}", nor its minimized variant "${apply}"`);
    }
  }
  defaultLocale = apply;
}
function getLocaleData(localeCodes, opts) {
  var _opts$localeMatcher;
  const requestedLocales = CanonicalizeLocaleList(localeCodes);
  const opt = Object.create(null);
  opt.localeMatcher = (_opts$localeMatcher = opts.localeMatcher) !== null && _opts$localeMatcher !== void 0 ? _opts$localeMatcher : 'best fit';
  opt.nu = opts.numberingSystem;
  const {
    dataLocale
  } = ResolveLocale(availableLocales, requestedLocales, opt, ['nu'], localeData, getDefaultLocale);
  return localeData[dataLocale];
}
function supportedLocalesOf(locales, options) {
  return SupportedLocales(availableLocales, CanonicalizeLocaleList(locales), options);
}
function addLocaleData(locale, data) {
  const minimized = String(new Intl.Locale(locale).minimize());
  localeData[locale] = data;
  localeData[minimized] = data;
  availableLocales.add(locale).add(minimized);
  if (defaultLocale == null) defaultLocale = minimized;
}

/**
 * Extracted from @formatjs/intl because their ESM building is broken and we
 * cannot reliably use this (`getFormatter`) function.
 *
 * @license MIT
 *   https://github.com/formatjs/formatjs/blob/fbc878f973a997c5cba330e1e72bf5219e3f1a67/packages/intl/LICENSE.md
 */
const NUMBER_FORMAT_OPTIONS = ['style', 'currency', 'currencyDisplay', 'unit', 'unitDisplay', 'useGrouping', 'minimumIntegerDigits', 'minimumFractionDigits', 'maximumFractionDigits', 'minimumSignificantDigits', 'maximumSignificantDigits', 'compactDisplay', 'currencyDisplay', 'currencySign', 'notation', 'signDisplay', 'unit', 'unitDisplay', 'numberingSystem'];
function getFormatter(_ref, getNumberFormat, options) {
  let {
    locale,
    formats,
    onError
  } = _ref;
  if (options === void 0) {
    options = {};
  }
  const {
    format
  } = options;
  const defaults = format && getNamedFormat(formats, 'number', format, onError) || {};
  const filteredOptions = filterProps(options, NUMBER_FORMAT_OPTIONS, defaults);
  return getNumberFormat(locale, filteredOptions);
}

const CompactNumberSym = Symbol('isCompactNumber');
function toFakeInternalSlots(nf) {
  const opts = nf.resolvedOptions();
  const localeData = getLocaleData(nf.resolvedOptions().locale, opts);
  if (localeData == null) {
    throw new Error(`Missing locale data for locale "${opts.locale}"`);
  }
  return {
    ...nf.resolvedOptions(),
    dataLocaleData: localeData
  };
}
function formatCompactNumber(getNumberFormat, config, value, options) {
  let nf;
  const {
    onError,
    locale,
    formats
  } = config;
  try {
    nf = getFormatter({
      locale,
      formats,
      onError
    }, getNumberFormat, {
      ...(options !== null && options !== void 0 ? options : {}),
      notation: 'compact'
    });
  } catch (e) {
    onError == null ? void 0 : onError(new IntlError(IntlErrorCode.FORMAT_ERROR, 'Error creating formatter for the compact number.', e));
    nf = undefined;
  }
  const reportError = err => config.onError(new IntlError(IntlErrorCode.FORMAT_ERROR, 'Error formatting the compact number', err));
  const ensureFormatter = () => {
    if (nf == null) throw new Error('Formatter is not initialized');
    return nf;
  };
  let roundedValue;
  let formattedValue;
  let parts;
  return {
    [CompactNumberSym]: true,
    valueOf() {
      if (roundedValue == null) {
        try {
          const nf = ensureFormatter();
          const internalSlots = toFakeInternalSlots(nf);
          const [exponent] = ComputeExponent(nf, value, {
            getInternalSlots() {
              return internalSlots;
            }
          });
          const numeric = exponent < 0 ? value * Math.pow(10, -exponent) : value / Math.pow(10, exponent);
          const {
            roundedNumber
          } = FormatNumericToString(nf.resolvedOptions(), numeric);
          roundedValue = roundedNumber * Math.pow(10, exponent);
        } catch (err) {
          reportError(err);
          roundedValue = value;
        }
      }
      return roundedValue;
    },
    toString() {
      if (formattedValue == null) {
        try {
          formattedValue = ensureFormatter().format(value);
        } catch (err) {
          reportError(err);
          formattedValue = String(value);
        }
      }
      return formattedValue;
    },
    toParts() {
      if (parts == null) {
        try {
          parts = ensureFormatter().formatToParts(value);
        } catch (err) {
          reportError(err);
          parts = [{
            type: 'literal',
            value: String(value)
          }];
        }
      }
      return parts;
    }
  };
}
function isCompactNumber(value) {
  return typeof value === 'object' && value != null && CompactNumberSym in value && value[CompactNumberSym] === true;
}

function formatCustomMessage(config, getMessageFormat, message, values, opts) {
  try {
    return getMessageFormat(message, config.locale, config.formats, opts).format(values);
  } catch (err) {
    let messageSlice;
    if (typeof message === 'string') {
      messageSlice = message.slice(0, 20);
      if (message.length > 20) {
        messageSlice += '...';
      }
    } else {
      messageSlice = '<compiled message>';
    }
    config.onError == null ? void 0 : config.onError(new IntlError(IntlErrorCode.FORMAT_ERROR, `Error formatting custom message "${messageSlice}".`, err));
  }
  return '';
}

const SECOND = 1000;
const MINUTE = 60 * SECOND;
const HOUR = 60 * MINUTE;
const DAY = 24 * HOUR;
const WEEK = 7 * DAY;
const MONTH = 30 * DAY;
const QUARTER = 3 * MONTH;
const YEAR = 365 * DAY;
const matchers = [['year', YEAR], ['quarter', QUARTER], ['month', MONTH], ['week', WEEK], ['day', DAY], ['hour', HOUR], ['minute', MINUTE], ['second', 0, SECOND]];
function normalizeUnit(unit) {
  for (const [knownUnit] of matchers) {
    if (knownUnit + 's' === unit) return knownUnit;
  }
  return unit;
}
function toTimestamp(value) {
  if (value instanceof Date) {
    return value.getTime();
  }
  if (typeof value === 'string') {
    return new Date(value).getTime();
  }
  return Number(value);
}
function extractRangePoints(range) {
  let from;
  let to;
  if (typeof range === 'object') {
    if (range instanceof Date) {
      from = toTimestamp(range);
      to = Date.now();
    } else {
      from = toTimestamp(range.from);
      if (range.to == null) {
        to = Date.now();
      } else {
        to = toTimestamp(range.to);
      }
    }
  } else {
    from = toTimestamp(range);
    to = Date.now();
  }
  return [from, to];
}
function getExcludedUnits(options) {
  const excludedUnits = [];
  const optionsExcludedUnits = options == null ? void 0 : options.excludedUnits;
  if (optionsExcludedUnits != null) {
    if (Array.isArray(optionsExcludedUnits)) {
      for (const unit of optionsExcludedUnits) excludedUnits.push(unit);
    } else {
      throw new TypeError('Value is not of array type for formatTimeDifference options property excludedUnits');
    }
  } else {
    excludedUnits.push('quarter');
  }
  return excludedUnits;
}
function filterMatchers(options) {
  let filteredMatchers;
  const excludedUnits = getExcludedUnits(options);
  if (excludedUnits.length === 0) {
    filteredMatchers = matchers;
  } else {
    filteredMatchers = [...matchers];
    for (const unit of excludedUnits) {
      const normalizedUnit = normalizeUnit(unit);
      const matcherToExcludeIndex = filteredMatchers.findIndex(matcher => matcher[0] === normalizedUnit);
      if (matcherToExcludeIndex === -1) {
        throw new RangeError(`Value ${unit} out of range for formatTimeDifference options property excludedUnits`);
      }
      filteredMatchers.splice(matcherToExcludeIndex, 1);
    }
  }
  return filteredMatchers;
}
function getMinimumMaximumUnits(options) {
  var _options$minimumUnit, _options$maximumUnit;
  let minUnit = (_options$minimumUnit = options == null ? void 0 : options.minimumUnit) !== null && _options$minimumUnit !== void 0 ? _options$minimumUnit : 'none';
  let maxUnit = (_options$maximumUnit = options == null ? void 0 : options.maximumUnit) !== null && _options$maximumUnit !== void 0 ? _options$maximumUnit : 'none';
  if (minUnit !== 'none') {
    minUnit = normalizeUnit(String(minUnit));
  }
  if (maxUnit !== 'none') {
    maxUnit = normalizeUnit(String(maxUnit));
  }
  return [minUnit, maxUnit];
}
function calculateBoundaries(filteredMatchers, minimumUnit, maximumUnit) {
  const maximumUnitMatcherIndex = maximumUnit === 'none' ? 0 : filteredMatchers.findIndex(matcher => matcher[0] === maximumUnit);
  let minimumUnitMatcherIndex;
  if (minimumUnit === 'none') {
    minimumUnitMatcherIndex = filteredMatchers.length > 0 ? filteredMatchers.length - 1 : 0;
  } else {
    minimumUnitMatcherIndex = filteredMatchers.findIndex(matcher => matcher[0] === minimumUnit);
  }
  const minimumUnitGreaterThanMaximumUnit = minimumUnitMatcherIndex < maximumUnitMatcherIndex;
  const minimumUnitOutOfRange = minimumUnit !== 'none' && minimumUnitMatcherIndex === -1;
  const maximumUnitOutOfRange = maximumUnit !== 'none' && maximumUnitMatcherIndex === -1;
  if (minimumUnitGreaterThanMaximumUnit || minimumUnitOutOfRange || maximumUnitOutOfRange) {
    let invalidValue;
    let invalidProperty;
    if (minimumUnitOutOfRange || minimumUnitGreaterThanMaximumUnit) {
      invalidValue = minimumUnit;
      invalidProperty = 'minimumUnit';
    } else if (maximumUnitOutOfRange) {
      invalidValue = maximumUnit;
      invalidProperty = 'maximumUnit';
    }
    throw new RangeError(`Value ${invalidValue} out of range for formatTimeDifference options property ${invalidProperty}`);
  }
  if (minimumUnitMatcherIndex === -1) {
    minimumUnitMatcherIndex = filteredMatchers.length - 1;
  }
  return [minimumUnitMatcherIndex, maximumUnitMatcherIndex];
}
function tryFormattingAsRelativeTime(formatRelativeTime, from, to, options) {
  const filteredMatchers = filterMatchers(options);
  if (filteredMatchers.length === 0) return null;
  const [minimumUnit, maximumUnit] = getMinimumMaximumUnits(options);
  const [minimumUnitMatcherIndex, maximumUnitMatcherIndex] = calculateBoundaries(filteredMatchers, minimumUnit, maximumUnit);
  const diff = to - from;
  const diffAbs = Math.abs(diff);
  for (let currentMatcherIndex = Math.max(maximumUnitMatcherIndex - 1, 0); currentMatcherIndex <= minimumUnitMatcherIndex; currentMatcherIndex++) {
    const matcher = filteredMatchers[currentMatcherIndex];
    if (diffAbs < matcher[1]) continue;
    if (currentMatcherIndex < maximumUnitMatcherIndex) break;
    const roundedDivision = Math.round(diffAbs / matcher[matcher.length > 2 ? 2 : 1]);
    return formatRelativeTime(diff < 0 ? roundedDivision : -roundedDivision, matcher[0], {
      numeric: 'auto',
      ...options
    });
  }
  return null;
}
function formatTimeDifference(_ref, formatRelativeTime, formatDate, range, options) {
  let {
    onError
  } = _ref;
  const reportError = err => onError(new IntlError(IntlErrorCode.FORMAT_ERROR, 'Error formatting time difference.', err));
  let from, to;
  try {
    ;
    [from, to] = extractRangePoints(range);
  } catch (err) {
    reportError(err);
    return '';
  }
  try {
    const relative = tryFormattingAsRelativeTime(formatRelativeTime, from, to, options);
    if (relative != null) return relative;
  } catch (err) {
    reportError(err);
  }
  try {
    var _options$dateTimeOpti;
    return formatDate(from, (_options$dateTimeOpti = options == null ? void 0 : options.dateTimeOptions) !== null && _options$dateTimeOpti !== void 0 ? _options$dateTimeOpti : {
      dateStyle: 'long',
      timeStyle: 'short'
    });
  } catch (err) {
    reportError(err);
  }
  return '';
}

function createIntl(config, cache) {
  const shape = createIntl$1(config, cache);
  if (supportedLocalesOf(config.locale).length < 1) {
    shape.onError(new MissingDataError(`Missing locale data for locale: "${config.locale}" of compact number API.`));
  }
  shape.formatCompactNumber = formatCompactNumber.bind(null, shape.formatters.getNumberFormat, shape);
  shape.formatCustomMessage = formatCustomMessage.bind(null, shape, shape.formatters.getMessageFormat);
  const formatTimeDifference$1 = formatTimeDifference.bind(null, shape, shape.formatRelativeTime, shape.formatDate);
  shape.formatTimeDifference = formatTimeDifference$1;
  shape.$ago = formatTimeDifference$1;
  const formatMessage = shape.formatMessage;
  function normalized$formatMessage(descriptor, values, opts) {
    const result = formatMessage(descriptor, values, opts);
    if (Array.isArray(result)) {
      for (let i = 0, l = result.length; i < l; i++) {
        const part = result[i];
        if (isCompactNumber(part)) {
          result[i] = String(part);
        }
      }
      if (result.every(part => typeof part === 'string')) {
        return result.join('');
      }
    }
    return result;
  }
  shape.$t = normalized$formatMessage;
  shape.formatMessage = normalized$formatMessage;
  return shape;
}

export { addLocaleData as addNumberFormatData, createIntl, setDefaultLocaleDataLocale as setDefaultNumberFormatDataLocale };
